@@ -42,7 +42,6 @@ twostagelvm <- function(object, model2,
     return(res)
 }
 
-
 uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
     if (!is.function(nlobj)) {
         predict.fun <- lapply(nlobj, function(x) x[["pred"]])
@@ -80,7 +79,6 @@ uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
     return(cbind(predict.fun(P$mean, P$var, model.frame(model1))))
 }
 
-
 ##' Two-stage estimator
 ##'
 ##' Generic function.
@@ -89,7 +87,7 @@ uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
 ##' @export
 ##' @param object Model object
 ##' @param ... Additional arguments to lower level functions
-"twostage" <- function(object,...) UseMethod("twostage")
+"twostage" <- function(object, ...) UseMethod("twostage")
 
 ##' Two-stage estimator (non-linear SEM)
 ##'
@@ -139,6 +137,7 @@ uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
 ##'     plot(mm,f=pf,data=data.frame(u=seq(-2,2,length.out=100)),lwd=2)
 ##' }
 ##'
+##' \donttest{ ## Reduce test timing
 ##' ## Splines
 ##' f <- function(x) cos(2*x)+x+-0.25*x^2
 ##' m <- lvm(x1+x2+x3~eta1, y1+y2+y3~eta2, latent=~eta1+eta2)
@@ -173,6 +172,7 @@ uhat <- function(p=coef(model1), model1, data=model.frame(model1), nlobj) {
 ##'   lines(x,p1[,1],col="green",lwd=5)
 ##'   confband(x,lower=p1[,2],upper=p1[,3],center=p1[,1], polygon=TRUE, col=Col(3,0.2))
 ##' }
+##' } ## Reduce test timing
 ##'
 ##' \dontrun{ ## Reduce timing
 ##'  ## Cross-validation example
@@ -220,18 +220,17 @@ twostage.lvmfit <- function(object, model2, data=NULL,
         if (is.null(id2)) id2 <- seq(nrow(model.frame(model2)))
         model1 <- object
         if (!inherits(object,"estimate")) {
-            model1 <- estimate(NULL,coef=p1,id=id1,iid=iid(object))
+            model1 <- estimate(NULL, coef=p1, id=id1, IC=IC(object))
         }
 
         e2 <- estimate(model2, id=id2)
-        U <- function(alpha=p1,beta=p2) {
-            pp <- uhat(alpha,object,nlobj=val$nonlinear)
+        U <- function(alpha = p1, beta = p2) {
+            pp <- uhat(alpha, object, nlobj = val$nonlinear)
             newd <- model.frame(model2)
-            newd[,colnames(pp)] <- pp
-            score(model2,p=beta,data=newd)
+            newd[, colnames(pp)] <- pp
+            score(model2, p = beta, data = newd)
         }
-        Ia <- numDeriv::jacobian(function(p) U(p),p1)
-        stacked <- stack(model1,e2,Ia)
+        stacked <- stack(model1,e2,U=U)
     } else {
         e2 <- estimate(coef=p2,vcov=NA)
     }
@@ -278,7 +277,6 @@ estimate.twostage.lvm <- function(x,data,...) {
 ##' @export
 twostage.twostage.lvm <- function(object,...) estimate.twostage.lvm(object,...)
 
-
 ##' @export
 twostage.lvm <- function(object,model2,data=NULL, ...) {
     if (is.null(data)) {
@@ -298,13 +296,12 @@ twostage.estimate <- twostage.lvmfit
 
 ##' @export
 print.twostage.lvm <- function(x,...) {
-    printline()
+    print(cli::rule())
     cat("Model 1:\n")
     print(Model(x$call$object))
-    printline()
+    print(cli::rule())
     cat("Model 2:\n")
     print(Model(x$call$model2))
-
 }
 
 ##' @export
@@ -429,7 +426,7 @@ predict.twostage.lvmfit <- function(object,
 ##' @param nfolds Number of folds (cross-validation)
 ##' @param rep Number of repeats of cross-validation
 ##' @param messages print information (>0)
-##' @param ... additional arguments to lower level functions
+##' @param ... additional arguments to lower
 ##' @examples
 ##' \donttest{ ## Reduce Ex.Timings##'
 ##' m1 <- lvm( x1+x2+x3 ~ u, latent= ~u)
@@ -478,11 +475,17 @@ twostageCV <- function(model1, model2, data, control1=list(trace=0), control2=li
     control1$start <- NULL
     ee <- list(e1a)
 
-    nmix <- setdiff(nmix,1)
-    val <- future_lapply(as.list(nmix), function(k) {
-      if (messages>0) cat("Fitting mixture model with", k, "components\n")
-      mixture(model1, k=k, data=data, control=c(control1,list(start=startf(k))))
-    }, ...)
+    nmix <- setdiff(nmix, 1)
+    fit <- function(k, ...) {
+        if (messages > 0) cat("Fitting mixture model with", k, "components\n")
+        mixture(model1, k = k, data = data, control = c(control1, list(start = startf(k))))
+    }
+    future.args <- list(...)
+    if (is.null(future.args$future.seed)) {
+      future.args$future.seed <- TRUE
+    }
+    args <-  c(list(FUN = fit, c(as.list(nmix)), future.args))
+    val <- do.call(future_lapply, args)
     ee <- c(ee, val)
     AIC1 <- unlist(lapply(ee,AIC))
     names(AIC1) <- c(1,nmix)
@@ -508,23 +511,22 @@ twostageCV <- function(model1, model2, data, control1=list(trace=0), control2=li
     ff <- lapply(MM,
                  function(m) function(data,e0,...)  twostage(e0,m,data=data,std.derr=FALSE))
     a <- cv(ff, data=data, K=nfolds, rep=rep, shared=f0)
-    M <- MM[[which.min(coef(a))]]
+    M <- MM[[which.min(a$coef)]]
     e2 <- twostage(e1, M, data, control=control2, std.err=std.err)
     options(op)
-    res <- list(AIC1=cbind(AIC1), model1=e1, model2=e2, cv=coef(a), knots=c(list(NA),Knots),
+    res <- list(AIC1=cbind(AIC1), model1=e1, model2=e2, cv=a$coef, knots=c(list(NA), Knots),
                 nfolds=nfolds, rep=rep)
     structure(res, class="twostageCV")
 }
 
-
 ##' @export
 print.twostageCV <- function(x,...) {
-    printline(70)
+    print(cli::rule())
     i1 <- which.min(x$AIC1)
     nmix <- rownames(x$AIC1)[i1]
     cat("Selected mixture model: ",nmix," component", ifelse(i1>1, "s",""),"\n", sep="")
     print(x$AIC1)
-    printline(70)
+    print(cli::rule())
     i2 <- which.min(x$cv)
     splinedf <- unlist(lapply(x$knots,function(x) if (any(is.na(x))) return(1) else length(x)-1))
     cat("Selected spline model degrees of freedom: ", splinedf[i2] ,"\n", sep="")
@@ -535,7 +537,7 @@ print.twostageCV <- function(x,...) {
     rownames(rmse) <- paste0("df:",splinedf)
     colnames(rmse) <- paste0("RMSE(nfolds=",x$nfolds,", rep=",x$rep,")")
     print(rmse)
-    printline(70)
+    print(cli::rule())
     cat("\n")
     if (!inherits(x$model2, "lvmfit")) {
         print(x$model2$coefmat, quote=FALSE)
@@ -549,19 +551,21 @@ coef.twostageCV <- function(object,...) {
     coef(Model(object),...)
 }
 
+##' @export
 vcov.twostageCV <- function(object,...) {
     vcov(Model(object),...)
 }
 
-iid.twostageCV <- function(object,...) {
-    iid(Model(object),...)
+##' @export
+IC.twostageCV <- function(x,...) {
+    IC(Model(x),...)
 }
 
+##' @export
 Model.twostageCV <- function(x,...) {
     x$model2
 }
 
-
 ##' @export
 summary.twostageCV <- function(object,...) {
     res <- with(object, list(model1=summary(model1),
@@ -575,8 +579,6 @@ print.summary.twostageCV <- function(x,...) {
     print.twostageCV(x, ...)
 }
 
-
-
 ##' @export
 predict.twostageCV <- function(object,... ) {
     predict(Model(object),...)
